from django.test import TestCase

from djspoofer import utils


class UtilTests(TestCase):
    """
    Utility Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.test_str = 'Lorem ipsum dolor sit amet, consectetur adipiscing elit.'

    def test_spoof_session(self):
        test_proxy = 'user123:password456@example.com:4582'
        user_agent = 'My User Agent 1.0'
        with utils.SpoofSession(proxy_str=test_proxy, user_agent=user_agent) as session:
            self.assertEquals(session.proxies['http://'], f'http://{test_proxy}/')
            self.assertEquals(session.proxies['https://'], f'https://{test_proxy}/')

            self.assertEquals(session.headers['User-Agent'], user_agent)

            new_headers = {
                'accept': 'application/json',
                'accept-encoding': 'gzip, deflate, br',
                'empty-header': '',
                'accept-language': 'en-US,en;q=0.9',
            }
            session.update_headers(new_headers)

            self.assertDictEqual(
                dict(session.headers),
                {
                    'accept': 'application/json',
                    'accept-encoding': 'gzip, deflate, br',
                    'connection': 'keep-alive',
                    'user-agent': 'My User Agent 1.0',
                    'accept-language': 'en-US,en;q=0.9'
                }
            )

    def test_fake_profile(self):
        old_profile = utils.FakeProfile()
        profile = utils.FakeProfile()
        self.assertNotEquals(old_profile, profile)

        self.assertIn(profile.gender, ['M', 'F'])
        self.assertIn(profile.full_gender, ['MALE', 'FEMALE'])
        self.assertEquals(profile.full_name, f'{profile.first_name} {profile.last_name}')

        dob = profile.dob
        self.assertEquals(profile.dob_yyyymmdd, f'{dob.year}-{dob.month:02}-{dob.day:02}')
        self.assertTrue(profile.us_phone_number.startswith('+1'))
        self.assertEquals(len(profile.us_phone_number), 12)



import logging
import random
import string

from faker.providers import BaseProvider

from djstarter import utils
from . import dictionary

logger = logging.getLogger(__name__)


class UsernameProvider(BaseProvider):
    """
        Provider to generate human-looking usernames

        from faker import Faker
        from .providers import UsernameProvider
        fake = Faker()
        fake.add_provider(UsernameProvider)
        username = fake.username()
    """

    __provider__ = 'username'
    __lang__ = 'en_US'

    words = []
    MIN_LEN = 5
    MAX_LEN = 20
    VOWELS = 'aeiou'
    SEPARATORS = ('_', '-', 'x', 'X', 'o', 'O', '0', '1')
    CONJUNCTIONS = ('-and-', '_and_', '-or-', '_or_', '-with-', '_with_', '-nor-', '_nor_', '-for-', '_for_')
    HONORIFICS = ('Mr', 'Mister', 'Ms', 'Miss', 'Mrs', 'Master', 'Sir', 'Madam', 'Dame', 'Lord', 'Lady', 'Dr',
                  'Prof', 'Sr', 'Fr', 'Rev', 'Elder', 'Judge', 'Duke')

    def __new__(cls, *args, **kw):
        if not hasattr(cls, '_instance'):   # pragma: no cover
            cls.words = cls.load_words()
            cls._instance = super().__new__(cls)
        return cls._instance

    @property
    def rand_word(self):
        return random.choice(self.words)

    def username(self):
        word = self.rand_word
        alg_map = {
            self.repeat: (word,),
            self.replace: (word,),
            self.concat: (word, self.rand_word),
            self.vowel_remove: (word,),
            self.concat_digits: (word,),
            self.decorate: (word,),
            self.leet: (word,),
            self.honorific: (word,),
        }
        alg = random.choice(list(alg_map.keys()))
        args = alg_map[alg]
        new_word = alg(*args)[:self.MAX_LEN]
        return self.reverse(word) if new_word == word else new_word

    @classmethod
    def load_words(cls):
        return [word.strip() for word in dictionary.ENGLISH_WORDS if (cls.MIN_LEN <= len(word) <= cls.MAX_LEN)]

    @classmethod
    def reverse(cls, word):
        return word[::-1]

    @classmethod
    def repeat(cls, word, chance=.1):
        return ''.join(i + i if utils.dice_roll(chance) else i for i in word)

    @classmethod
    def replace(cls, word, chance=.1):
        new_letters = []
        for i, letter in enumerate(word):
            if utils.dice_roll(chance):
                new_letters.append(random.choice(string.ascii_lowercase.replace(letter, '')))
            else:
                new_letters.append(letter)
        return ''.join(new_letters)

    @classmethod
    def concat(cls, word1, word2, chance=.3):
        word1 = word1.capitalize() if utils.dice_roll(chance) else word1
        word2 = word2.capitalize() if utils.dice_roll(chance) else word2
        space = random.choice(cls.SEPARATORS+cls.CONJUNCTIONS)
        return f'{word1}{space}{word2}'

    @classmethod
    def vowel_remove(cls, word, chance=.1):
        return ''.join('' if utils.dice_roll(chance) and i in cls.VOWELS else i for i in word)

    @classmethod
    def concat_digits(cls, word, chance=.5):
        digits = str(generate_digits(random.randint(1, 2)))
        return word + digits if utils.dice_roll(chance) else digits + word

    @classmethod
    def decorate(cls, word, chance=.5, min_chars=1, max_chars=3):
        l_deco = ''.join(random.choice(cls.SEPARATORS) for _ in range(random.randint(min_chars, max_chars)))
        r_deco = l_deco[::-1] if utils.dice_roll(chance) else ''  # Slicing syntax means reverse string
        return f'{l_deco}{word}{r_deco}'

    @classmethod
    def leet(cls, text, chance=.33):
        leet_map = {
            'o': '0',
            'i': '1',
            'e': '3',
            'a': '4',
            's': '5',
            't': '7',
        }

        for symbol, replaceStr in leet_map.items():
            if utils.dice_roll(chance):
                text = text.replace(symbol, replaceStr)
                text = text.replace(symbol.upper(), replaceStr)
        return text

    @classmethod
    def honorific(cls, word, chance=.5):
        h = random.choice(cls.HONORIFICS)
        h = h.title() if utils.dice_roll(chance) else h.lower()
        return f'{h}{random.choice(cls.SEPARATORS)}{word}'


class PhoneNumberProvider(BaseProvider):
    """
        Provider to generate human-looking usernames

        from faker import Faker
        from .providers import UsernameProvider
        fake = Faker()
        fake.add_provider(PhoneNumberProvider)
        username = fake.username()
    """

    __provider__ = 'phone_number'
    __lang__ = 'en_US'

    @classmethod
    def us_e164(cls):
        return f'+1{generate_digits(10)}'


def generate_digits(n_digits):
    return random.randint(10 ** (n_digits - 1), (10 ** n_digits) - 1)


def replace_character(orig_str, idx, char):
    return orig_str[:idx] + char + orig_str[idx + 1:]



from django.test import TestCase

from djspoofer.models import Profile, Proxy


class ProfileTests(TestCase):
    """
    Profile Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.profile_data = {
            'device_category': 'mobile',
            'platform': 'US',
            'screen_height': 1920,
            'screen_width': 1080,
            'user_agent': 'My User Agent 1.0',
            'viewport_height': 768,
            'viewport_width': 1024,
            'weight': .005,
        }

    def test_user_str(self):
        profile = Profile.objects.create(**self.profile_data)
        self.assertEqual(str(profile), f'Profile: {profile.user_agent}')


class ProxyTests(TestCase):
    """
    Proxy Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.proxy_data = {
            'url': 'user123:password456@example.com:4582',
            'country': 'US',
            'city': 'dallas',
        }

    def test_user_str(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertEqual(str(proxy), f'Proxy: {proxy.url} - {proxy.mode}')

    def test_on_cooldown(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertFalse(proxy.on_cooldown)

        proxy.set_last_used()
        self.assertTrue(proxy.on_cooldown)

    def test_set_last_used(self):
        proxy = Proxy.objects.create(**self.proxy_data)
        self.assertEquals(proxy.used_ct, 0)
        self.assertIsNone(proxy.last_used)

        proxy.set_last_used()
        self.assertEquals(proxy.used_ct, 1)
        self.assertIsNotNone(proxy.last_used)

    # def test_dupe_username(self):
    #     MarzUser.objects.create(**self.marz_user_data)
    #     with self.assertRaises(IntegrityError):
    #         MarzUser.objects.create(**self.marz_user_data)
    #
    # def test_marz_user_toggle_lock(self):
    #     marz_user = MarzUser.objects.create(**self.marz_user_data)
    #     marz_user.lock()
    #     self.assertTrue(marz_user.is_locked)
    #     marz_user.unlock()
    #     self.assertFalse(marz_user.is_locked)
    #     self.assertIsNone(marz_user.locked_at)
    #
    # def test_marz_user_update_session_info(self):
    #     marz_user = MarzUser.objects.create(**self.marz_user_data)
    #     self.assertEquals(marz_user.session_count, 0)
    #     marz_user.update_session_info()
    #     self.assertEquals(marz_user.session_count, 1)



from django.core.management.base import BaseCommand
from ... import tasks


class Command(BaseCommand):
    help = 'Get Intoli Profiles'

    def handle(self, *args, **kwargs):
        try:
            tasks.get_profiles()
        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL('Finished getting profiles from Intoli'))



from djstarter import utils as core_utils

BROWSER_REFRESH_MSG = 'Browser will refresh periodically to display the status.'


class ProxyModes(core_utils.ChoiceEnum):
    ROTATING = 0
    STICKY = 1
    GENERAL = 2



import datetime
import uuid

from django.db import models
from django.utils import timezone

from . import const, managers


class BaseModel(models.Model):
    oid = models.UUIDField(primary_key=True, default=uuid.uuid4)
    created = models.DateTimeField(auto_now_add=True)
    modified = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


class Profile(BaseModel):
    objects = managers.ProfileManager()

    device_category = models.CharField(max_length=16)
    platform = models.CharField(max_length=16)
    screen_height = models.IntegerField()
    screen_width = models.IntegerField()
    user_agent = models.TextField()
    viewport_height = models.IntegerField()
    viewport_width = models.IntegerField()
    weight = models.DecimalField(max_digits=25, decimal_places=24)

    class Meta:
        db_table = 'djspoofer_profile'
        ordering = ['-weight']
        app_label = 'djspoofer'

        indexes = [
            models.Index(fields=['device_category', ], name='profile_device_category_index'),
            models.Index(fields=['platform', ], name='profile_platform_index'),
        ]

    def __str__(self):
        return f'Profile: {self.user_agent}'


class Proxy(BaseModel):
    objects = managers.ProxyManager()

    url = models.TextField(unique=True, blank=False)
    mode = models.IntegerField(default=const.ProxyModes.GENERAL.value, choices=const.ProxyModes.choices())
    country = models.CharField(max_length=3, blank=True, null=True)
    city = models.CharField(max_length=64, blank=True, null=True)
    last_used = models.DateTimeField(blank=True, null=True)
    used_ct = models.IntegerField(default=0)
    cooldown = models.DurationField(default=datetime.timedelta(minutes=10))

    class Meta:
        db_table = 'djspoofer_proxy'
        ordering = ['url']
        app_label = 'djspoofer'

    def __str__(self):
        return f'Proxy: {self.url} - {self.mode}'

    @property
    def on_cooldown(self):
        if self.last_used:
            return self.last_used > timezone.now() - self.cooldown
        return False

    def set_last_used(self):
        self.last_used = timezone.now()
        self.used_ct += 1
        self.save()



from django.conf.locale.en import formats as en_formats
from django.contrib import admin

from djstarter import admin as core_admin
from . import models

en_formats.DATETIME_FORMAT = "M d y H:i"


@admin.register(models.Profile)
class ProfileAdmin(core_admin.BaseAdmin):
    list_display = ['created', 'device_category', 'platform', 'user_agent', 'weight']
    list_filter = ('device_category', 'platform',)
    ordering = ['-created']
    search_fields = ['user_agent']

    show_full_result_count = False

    def bulk_delete(self, request, queryset):
        queryset.delete()

    bulk_delete.short_description = 'Bulk Delete'

    actions = ['export_as_csv', bulk_delete]


@admin.register(models.Proxy)
class ProxyAdmin(core_admin.BaseAdmin):
    list_display = ['url', 'mode', 'cooldown', 'last_used', 'used_ct', 'country', 'city']
    list_filter = ('mode', 'country', 'city')
    ordering = ['mode', 'url']
    search_fields = ['url']

    show_full_result_count = False
    read_only_exclude = ('url', 'mode', 'country')

    def set_rotating(self, request, queryset):
        queryset.update(mode=models.Proxy.Modes.ROTATING.value)

    set_rotating.short_description = 'Set as Rotating'

    def set_sticky(self, request, queryset):
        queryset.update(mode=models.Proxy.Modes.STICKY.value)

    set_sticky.short_description = 'Set as Sticky'

    def set_general(self, request, queryset):
        queryset.update(mode=models.Proxy.Modes.GENERAL.value)

    set_general.short_description = 'Set as General'

    def bulk_delete(self, request, queryset):
        queryset.delete()

    bulk_delete.short_description = 'Bulk Delete'

    actions = ['export_as_csv', set_rotating, set_sticky, set_general, bulk_delete]



import gzip
import json
import logging
from io import BytesIO
from json.decoder import JSONDecodeError

from httpx import RequestError

from . import exceptions

logger = logging.getLogger(__name__)
retry_exceptions = (RequestError, JSONDecodeError)

UA_BAD_CHARS = '\"\''
UA_BLACKLIST = ('coc_coc_browser', 'QQBrowser', 'WeChat', 'YaBrowser', 'zh-CN', 'zhihu')
UA_MIN_LEN = 40
UA_MAX_LEN = 160


def is_valid_profile(profile):
    if not UA_MIN_LEN < len(profile.user_agent) < UA_MAX_LEN:
        return False
    elif any(x.lower() in profile.user_agent.lower() for x in UA_BLACKLIST):
        return False
    elif len(profile.device_category) > 16:
        return False
    elif len(profile.platform) > 16:
        return False
    else:
        return True


def get_profiles(s_client):
    url = 'https://raw.githubusercontent.com/intoli/user-agents/master/src/user-agents.json.gz'

    params = {
        'format': 'json',
    }

    with s_client.stream('GET', url, params=params) as response:
        if response.status_code >= 500:
            response.raise_for_status()
        if response.is_error:
            raise exceptions.DJSpooferError(info=f'Problem getting file at url: {url}')

        json_io = BytesIO()
        try:
            for chunk in response.iter_bytes(chunk_size=8192):
                json_io.write(chunk)
                json_io.flush()
            json_io.seek(0)
            with gzip.GzipFile(fileobj=json_io, mode='rb') as gz_file:
                raw_json = json.load(gz_file)
                raw_profiles = [IntoliProfile(profile) for profile in raw_json]
                logger.info(f'Number of Raw Intoli Profiles: {len(raw_profiles)}')
                return [p for p in raw_profiles if is_valid_profile(p)]
        finally:
            json_io.close()


class IntoliProfile:
    def __init__(self, js):
        self.device_category = js.get('deviceCategory')
        self.platform = js.get('platform')
        self.screen_height = js.get('screenHeight') or 1080
        self.screen_width = js.get('screenWidth') or 1920
        self.user_agent = js.get('userAgent').strip(UA_BAD_CHARS)
        self.viewport_height = js.get('viewportHeight') or 920
        self.viewport_width = js.get('viewportWidth') or 1415
        self.weight = js.get('weight')


import datetime

from django.test import TestCase
from django.utils import timezone

from djspoofer import const, exceptions
from djspoofer.models import Profile, Proxy


class ProfileManagerTests(TestCase):
    """
    ProfileManager Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.profile_data = {
            'device_category': 'mobile',
            'platform': 'US',
            'screen_height': 1920,
            'screen_width': 1080,
            'user_agent': 'My User Agent 1.0',
            'viewport_height': 768,
            'viewport_width': 1024,
            'weight': .005,
        }

    def test_all_user_agents(self):
        Profile.objects.create(**self.profile_data)

        new_data = self.profile_data.copy()
        new_data['user_agent'] = 'The 2nd User Agent 2.0'
        Profile.objects.create(**new_data)

        user_agents = Profile.objects.all_user_agents()

        self.assertListEqual(
            list(user_agents),
            ['My User Agent 1.0', 'The 2nd User Agent 2.0']
        )

    def test_rand_desktop_user_agent(self):
        with self.assertRaises(exceptions.DJSpooferError):
            Profile.objects.rand_desktop_user_agent()

        Profile.objects.create(**self.profile_data)

        with self.assertRaises(exceptions.DJSpooferError):
            Profile.objects.rand_desktop_user_agent()

        new_data = self.profile_data.copy()
        new_data['device_category'] = 'desktop'
        new_profile = Profile.objects.create(**new_data)

        user_agent = Profile.objects.rand_desktop_user_agent()

        self.assertEquals(new_profile.user_agent, user_agent)

    def test_weighted_desktop_user_agent(self):
        Profile.objects.create(**self.profile_data)

        with self.assertRaises(exceptions.DJSpooferError):
            Profile.objects.weighted_desktop_user_agent()

        new_data = self.profile_data.copy()
        new_data['device_category'] = 'desktop'
        new_profile = Profile.objects.create(**new_data)

        user_agent = Profile.objects.weighted_desktop_user_agent()

        self.assertEquals(new_profile.user_agent, user_agent)

    def test_desktop_profile_exists(self):
        Profile.objects.create(**self.profile_data)

        self.assertFalse(Profile.objects.desktop_profile_exists())

        new_data = self.profile_data.copy()
        new_data['device_category'] = 'desktop'
        Profile.objects.create(**new_data)

        self.assertTrue(Profile.objects.desktop_profile_exists())

    def test_older_than_n_minutes(self):
        profile = Profile.objects.create(**self.profile_data)

        self.assertFalse(Profile.objects.older_than_n_minutes().exists())

        profile.created = timezone.now() - datetime.timedelta(minutes=6)
        profile.save()

        self.assertTrue(Profile.objects.older_than_n_minutes().exists())


class ProxyManagerTests(TestCase):
    """
    ProxyManager Tests
    """

    @classmethod
    def setUpTestData(cls):
        cls.proxy_data = {
            'url': 'user123:password456@example.com:4582',
            'country': 'US',
            'city': 'dallas',
        }
        cls.proxy_data_2 = {
            'url': 'another123:password456@example.com:4582',
            'country': 'US',
            'city': 'dallas',
        }

    def test_get_rotating_proxy(self):
        Proxy.objects.create(**self.proxy_data)
        with self.assertRaises(Proxy.DoesNotExist):
            Proxy.objects.get_rotating_proxy()

        Proxy.objects.create(mode=const.ProxyModes.ROTATING.value, **self.proxy_data_2)
        self.assertIsNotNone(Proxy.objects.get_rotating_proxy())

    def test_get_sticky_proxy(self):
        Proxy.objects.create(**self.proxy_data)
        with self.assertRaises(Proxy.DoesNotExist):
            Proxy.objects.get_sticky_proxy()

        Proxy.objects.create(mode=const.ProxyModes.STICKY.value, **self.proxy_data_2)
        self.assertIsNotNone(Proxy.objects.get_sticky_proxy())

    def test_get_all_urls(self):
        Proxy.objects.create(**self.proxy_data)
        Proxy.objects.create(mode=const.ProxyModes.STICKY.value, **self.proxy_data_2)

        self.assertListEqual(
            sorted(list(Proxy.objects.get_all_urls())),
            sorted([self.proxy_data['url'], self.proxy_data_2['url']])
        )



from django.core.management.base import BaseCommand
from httpx import Client

from spoof import utils as spoof_utils
from ... import models


class Command(BaseCommand):
    help = 'Test Proxies'

    def add_arguments(self, parser):
        parser.add_argument(
            "--url",
            required=True,
            type=str,
            help="Target URL for proxies",
        )
        parser.add_argument(
            "--no-proxies",
            action='store_true',
            help="Omit proxies",
        )

    def handle(self, *args, **kwargs):
        url = kwargs['url']
        try:
            if kwargs['no_proxies']:
                with Client() as client:
                    client.get(url)
            else:
                with spoof_utils.SpoofSession(proxy_str=models.Proxy.objects.get_rotating_proxy().url) as client:
                    client.get(url)

        except Exception as e:
            self.stdout.write(self.style.ERROR(f'Error while running command:\n{str(e)}'))
            raise e
        else:
            self.stdout.write(self.style.MIGRATE_LABEL(f'Successful GET for "{url}"'))



import logging
import random
from collections import OrderedDict

from faker import Faker
from httpx import Client

from . import models, providers

logger = logging.getLogger(__name__)


class SpoofSession(Client):
    def __init__(self, proxy_str=None, user_agent=None):
        self.proxies = {
            'http://': f'http://{proxy_str}/',
            'https://': f'https://{proxy_str}/'
        }
        self.headers = {
            'User-Agent': user_agent or models.Profile.objects.weighted_desktop_user_agent()
        }
        super().__init__(proxies=self.proxies, headers=self.headers)

    def update_headers(self, new_headers):
        self.headers = OrderedDict({**self.headers, **self.clean_headers(new_headers)})

    @classmethod
    def clean_headers(cls, headers):
        return OrderedDict({k: v for k, v in headers.items() if v})


fake = Faker('en_US')
fake.add_provider(providers.UsernameProvider)
fake.add_provider(providers.PhoneNumberProvider)


class FakeProfile:
    MIN_PWD_LEN = 6

    def __init__(self, username=None):
        self.username = username or fake.username()
        self.gender = random.choice(['M', 'F'])
        self.first_name = fake.first_name_male() if self.gender == 'M' else fake.first_name_female()
        self.last_name = fake.last_name()
        self.dob = fake.date_of_birth(minimum_age=18, maximum_age=60)
        self.contact_email = f'{fake.username()}@{fake.free_email_domain()}'
        self.addr_state = fake.state_abbr()
        self.us_phone_number = fake.us_e164()
        self.password = fake.password(length=12, special_chars=True, digits=True, upper_case=True, lower_case=True)

    def __str__(self):
        return f'FakeProfile -> username: {self.username}, full_name: {self.full_name}'

    @property
    def full_gender(self):
        return 'MALE' if self.gender == 'M' else 'FEMALE'

    @property
    def full_name(self):
        return f'{self.first_name} {self.last_name}'

    @property
    def dob_yyyymmdd(self):
        return self.dob.strftime('%Y-%m-%d')



import datetime
import random

from django.db import models
from django.db import transaction
from django.db.models import F, Q
from django.utils import timezone

from . import const, exceptions


class ProfileManager(models.Manager):
    def all_user_agents(self):
        return super().get_queryset().values_list('user_agent', flat=True)

    def all_desktop_profiles(self):
        return super().get_queryset().filter(device_category='desktop')

    def rand_desktop_user_agent(self):
        if desktop_profiles := self.all_desktop_profiles():
            return random.choice(list(desktop_profiles)).user_agent
        raise exceptions.DJSpooferError('No Desktop Profiles Exist')

    def weighted_desktop_user_agent(self):
        if desktop_profiles := self.all_desktop_profiles():
            user_agents = [p.user_agent for p in desktop_profiles]
            weights = [float(p.weight) for p in desktop_profiles]
            return random.choices(population=user_agents, weights=weights, k=1)[0]

        raise exceptions.DJSpooferError('No Desktop Profiles Exist')

    def desktop_profile_exists(self):
        q = Q(device_category='desktop')
        return super().get_queryset().filter(q).exists()

    def older_than_n_minutes(self, minutes=5):
        q = Q(created__lt=timezone.now() - datetime.timedelta(minutes=minutes))
        return super().get_queryset().filter(q)


class ProxyManager(models.Manager):
    def get_rotating_proxy(self):
        q_filter = Q(mode=const.ProxyModes.ROTATING.value)
        return super().get_queryset().get(q_filter)

    def get_sticky_proxy(self):
        with transaction.atomic():
            q = Q(mode=const.ProxyModes.STICKY.value)
            q &= (Q(last_used__lt=timezone.now() - F('cooldown')) | Q(last_used=None))

            sticky_proxy = super().get_queryset().select_for_update(skip_locked=True).order_by(
                F('last_used').asc(nulls_first=True)).get(q)

            sticky_proxy.set_last_used()
            return sticky_proxy

    def get_all_urls(self):
        return super().get_queryset().values_list('url', flat=True)



import logging
from json import JSONDecodeError

from httpx import Client, RequestError
from urllib3.exceptions import HTTPError

from djspoofer import exceptions
from . import models, intoli

logger = logging.getLogger(__name__)

_warn_exceptions = (
    exceptions.ApiError,
    JSONDecodeError,
    RequestError,
    HTTPError
)


def get_profiles(*args, **kwargs):
    GetProfiles(*args, **kwargs).start()


class GetProfiles:
    def __init__(self, *args, **kwargs):
        pass

    @staticmethod
    def start():
        with Client() as client:
            profiles = intoli.get_profiles(client)
        new_profiles = []

        for profile in profiles:
            new_profiles.append(
                models.Profile(
                    device_category=profile.device_category,
                    platform=profile.platform,
                    screen_height=profile.screen_height,
                    screen_width=profile.screen_width,
                    user_agent=profile.user_agent,
                    viewport_height=profile.viewport_height,
                    viewport_width=profile.viewport_width,
                    weight=profile.weight,
                )
            )

        logger.info(f'Got {len(new_profiles)} New Intoli Profiles')
        try:
            models.Profile.objects.bulk_create(new_profiles)
        except Exception as e:
            raise exceptions.AppError(info=f'Error adding user agents: {str(e)}')
        else:
            logger.info(f'Deleted {models.Profile.objects.older_than_n_minutes().delete()[0]} Old Intoli Profiles')
